///File : DialogBarCodeRead.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "DialogBarCodeRead.hpp"


#if defined USE_BAR_CODE_MODE
//------------------------------------------------------------------------------
//-                               DialogBarCodeRead                                   -
//------------------------------------------------------------------------------
DialogBarCodeRead::DialogBarCodeRead( wxWindow* parent, wxWindowID id, ItemSectionDBBase *IRDBManager, const wxString &title, const wxPoint& pos, const wxSize& size, long style ) : wxDialog( parent, id, title, pos, size, style )
{
    // Devis Manager
    m_IRDBManager = IRDBManager;

    // AllItemsList
    m_stdAllItemsList = m_IRDBManager->GetAllItemsList();

	// Controls
        // Ref
	wxBoxSizer *MainSizer = new wxBoxSizer( wxVERTICAL );

    wxBoxSizer *RefSizer = new wxBoxSizer( wxHORIZONTAL );

    RefSizer->Add(new wxStaticText(this, wxID_ANY, _("Reference :")), 0, wxALIGN_CENTER | wxALL, 5);

	m_TextRef = new wxTextCtrl(this, wxID_ANY, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER );
	m_TextRef->SetFocus();
	RefSizer->Add(m_TextRef, 1, wxALL, 5);

	m_btAdd = new wxButton(this, wxID_ADD);
	RefSizer->Add(m_btAdd, 0, wxALL, 5);

	MainSizer->Add(RefSizer, 0, wxEXPAND);


        // List
    wxBoxSizer *ListBtSizer = new wxBoxSizer( wxHORIZONTAL );

    m_selectedItemsList = new wxListBox(this, wxID_ANY);
    ListBtSizer->Add(m_selectedItemsList, 1, wxEXPAND | wxALL, 5);

    m_btRemove = new wxButton(this, wxID_REMOVE);
    ListBtSizer->Add(m_btRemove, 0, wxALL, 5);

    MainSizer->Add(ListBtSizer, 1, wxEXPAND);

    	// Bts
    MainSizer->Add(CreateButtonSizer(wxOK | wxCANCEL), 0, wxALL | wxALIGN_RIGHT, 5);

	SetSizer(MainSizer);

	// Events
    m_TextRef->Connect( wxEVT_COMMAND_TEXT_ENTER, wxEventHandler(DialogBarCodeRead::OnAdd), NULL, this);
    m_TextRef->Connect( wxEVT_COMMAND_TEXT_UPDATED, wxEventHandler(DialogBarCodeRead::OnTextEdit ), NULL, this);

    m_btAdd->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxEventHandler(DialogBarCodeRead::OnAdd ), NULL, this);

    m_btRemove->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxEventHandler(DialogBarCodeRead::OnRemove ), NULL, this);

    m_selectedItemsList->Connect( wxEVT_KEY_DOWN, wxKeyEventHandler(DialogBarCodeRead::OnListKeyDown ), NULL, this);

    // /!\ pas de 'm_SectionsList->Connect' ==> SegmentationFault quand on quitte avec wxGTK et wx2.8.9, après avoir utilisé l'event !!
    Connect(m_selectedItemsList->GetId(), wxEVT_COMMAND_LISTBOX_SELECTED, wxEventHandler(DialogBarCodeRead::OnListSelect));

    // Table
    FeedSelectedItemsList();
    RefreshBt();
}
//------------------------------------------------------------------------------
void DialogBarCodeRead::RefreshBt()
{
    m_btRemove->Enable(!m_selectedItemsList->IsEmpty() && m_selectedItemsList->GetSelection() != wxNOT_FOUND);

    bool isRefOk = false;
	for(int i = 0; i < m_stdAllItemsList.size(); i++)
	{
        if( m_stdAllItemsList[i] == StrWxToStd(m_TextRef->GetValue()))
        {
            isRefOk = true;
            break;
        }
    }
    m_btAdd->Enable(isRefOk);
}
//------------------------------------------------------------------------------
void DialogBarCodeRead::OnListSelect(wxEvent& event)
{
    RefreshBt();
}
//------------------------------------------------------------------------------
void DialogBarCodeRead::OnTextEdit(wxEvent& event)
{
    RefreshBt();
}
//------------------------------------------------------------------------------
void DialogBarCodeRead::OnAdd(wxEvent& event)
{
    std::string ref = StrWxToStd(m_TextRef->GetValue());

    // Si existe
	for(int i = 0; i < m_stdAllItemsList.size(); i++)
	{
        if( m_stdAllItemsList[i] == ref)
        {
            // Si le produit est déjà dans la liste :
            bool isInList = false;
            int pos = 0;
            for(pos = 0; pos < m_selectedItems.size(); pos++)
            {
                if(m_selectedItems[pos] == ref)
                {
                    isInList = true;
                    break;
                }
            }

            if(isInList)
            {
                m_selectedItemQuantities[pos]++;
            }
            else
            {
                m_selectedItems.push_back(ref);
                m_selectedItemQuantities.push_back(1);
            }

            m_TextRef->SetValue(_T(""));
            FeedSelectedItemsList();

            return;
        }
    }
}
//------------------------------------------------------------------------------
void DialogBarCodeRead::OnRemove(wxEvent& event)
{
    int selection = m_selectedItemsList->GetSelection();
    if(selection != wxNOT_FOUND && selection < m_selectedItems.size())
    {
        m_selectedItems.erase(m_selectedItems.begin() + selection);
        m_selectedItemQuantities.erase(m_selectedItemQuantities.begin() + selection);
        FeedSelectedItemsList();
    }
}
//------------------------------------------------------------------------------

void DialogBarCodeRead::FeedSelectedItemsList()
{
    wxWindowUpdateLocker winLock(m_selectedItemsList); // Bloque les redraw durant son temps de vie.

    //On met la liste à la bonne taille :
    while(m_selectedItemsList->GetCount() > m_selectedItems.size())
    {
        m_selectedItemsList->Delete(m_selectedItemsList->GetCount() - 1);
    }
    while(m_selectedItemsList->GetCount() < m_selectedItems.size())
    {
        m_selectedItemsList->Append(_T(""));
    }

    // On change les labels...
    for(int i = 0; i < m_selectedItems.size(); i++)
    {
        wxString tmp = StrStdToWx(m_selectedItems[i]) + wxString(_T(" * ")) + wxNbToStr(m_selectedItemQuantities[i]);

        if(m_selectedItemsList->GetString(i) != tmp) // ...Si il le faut
        {
            m_selectedItemsList->SetString(i, tmp);
        }
    }

    RefreshBt();
}
//------------------------------------------------------------------------------
std::vector<std::string>& DialogBarCodeRead::GetItemsListRef()
{
    //std::vector<std::string> &selectedItems = m_selectedItems;
	return m_selectedItems;
}
//------------------------------------------------------------------------------
std::vector<double>& DialogBarCodeRead::GetItemsListQuantity()
{
    return m_selectedItemQuantities;
}
//------------------------------------------------------------------------------
void DialogBarCodeRead::OnListKeyDown(wxKeyEvent& event)
{
    if(event.GetKeyCode() == WXK_DELETE)
    {
        wxCommandEvent tmp;
        OnRemove(tmp);
    }

    event.Skip();
}
//------------------------------------------------------------------------------

#endif // #if defined USE_BAR_CODE_MODE
