///File : DialogMdoEdit.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "DialogMdoEdit.hpp"
//------------------------------------------------------------------------------
//-                               DialogMdoEdit                                   -
//------------------------------------------------------------------------------
DialogMdoEdit::DialogMdoEdit( wxWindow* parent, wxWindowID id, ConfigManager *generalCfg, const std::string &mainMdoSizerName, const wxString &title, const wxPoint& pos, const wxSize& size, long style ) : wxDialog( parent, id, title, pos, size, style )
{
    // Params
    m_generalCfg = generalCfg;
    m_mainMdoSizer = new mdo::Sizer(m_generalCfg, NULL, NULL, mainMdoSizerName);
    m_mainMdoSizer->LoadFromCfg();

    // Base
    m_draggedMdo = NULL;
    m_copyCutMdo = NULL;

    // mdoTypeNameList
    m_mdoTypeNameList.resize(6);
    m_mdoTypeNameList[0] = MDO_IMAGE_TYPE_NAME;
    m_mdoTypeNameList[1] = MDO_SIZER_TYPE_NAME "H";
    m_mdoTypeNameList[2] = MDO_SIZER_TYPE_NAME "V";
    m_mdoTypeNameList[3] = MDO_SPACER_TYPE_NAME;
    m_mdoTypeNameList[4] = MDO_STATIC_TEXT_TYPE_NAME;
    m_mdoTypeNameList[5] = MDO_DEVIS_TEXT_TYPE_NAME;

	// Controls
	wxBoxSizer *MainSizer = new wxBoxSizer( wxHORIZONTAL );

        // Draw Space
    m_drawSpaceSizer = new wxBoxSizer( wxVERTICAL );
    MainSizer->Add(m_drawSpaceSizer, 1, wxEXPAND | wxALL, 5);

        // Tree & autre
    wxBoxSizer *ControlsSizer = new wxBoxSizer( wxVERTICAL );

        // Tree
    m_tree = new wxTreeCtrl(this, wxID_ANY, wxDefaultPosition, wxSize(300, -1), wxTR_EDIT_LABELS | wxTR_HAS_BUTTONS);
    ControlsSizer->Add(m_tree, 1, wxEXPAND | wxALL, 5);

            // Img
    wxImageList *treeImgList = new wxImageList(32, 32);
    for(int i = 0; i < m_mdoTypeNameList.size(); i++)
    {
        wxString ImgFileName = findFile(wxString(_T("data/") + StrStdToWx(m_mdoTypeNameList[i]) + wxString(_T(".png"))));
        if(wxFileExists(ImgFileName))
        {
            wxImage img(ImgFileName, wxBITMAP_TYPE_PNG);
            treeImgList->Add(wxBitmap(img));
        }
    }
    m_tree->AssignImageList(treeImgList);

            // wxDropTarget
    /*MyDropTarget *treeDropTarget = new MyDropTarget();
    m_tree->SetDropTarget(treeDropTarget);*/

	MainSizer->Add(ControlsSizer, 0, wxEXPAND);

        // Bt
	wxGridSizer* btSizer = new wxGridSizer( 2 );

    m_btEdit = new wxButton(this, wxID_EDIT );
    btSizer->Add(m_btEdit, 0, wxEXPAND | wxALL, 5);

	m_btAdd = new wxButton(this, wxID_ADD);
	btSizer->Add(m_btAdd, 0, wxEXPAND | wxALL, 5);

	m_btRemove = new wxButton(this, wxID_REMOVE );
    btSizer->Add(m_btRemove, 0, wxEXPAND | wxALL, 5);

    m_btRename = new wxButton(this, wxID_ANY, _("&Rename") );
    btSizer->Add(m_btRename, 0, wxEXPAND | wxALL, 5);

    ControlsSizer->Add(btSizer, 0, wxEXPAND, 0);

        // Mode (Portrait / paysage)
    m_labelMode = new wxStaticText(this, wxID_ANY, _T(""));
    ControlsSizer->Add(m_labelMode, 0, wxEXPAND | wxALL, 5);

        // Fin
	SetSizer(MainSizer);
	SetMinSize(wxSize(500, 300));

        // Popup menu
    m_itemEdit = m_PopupMenu.Append(wxID_EDIT);
    m_itemAdd = m_PopupMenu.Append(wxID_ADD);
    m_itemRemove = m_PopupMenu.Append(wxID_REMOVE);
    m_itemRename = m_PopupMenu.Append(wxID_ANY, _("&Rename"));
    m_PopupMenu.AppendSeparator();
    m_ItemUp = m_PopupMenu.Append(wxID_UP);
    m_ItemDown = m_PopupMenu.Append(wxID_DOWN);
    m_PopupMenu.AppendSeparator();
    m_ItemCopy = m_PopupMenu.Append(wxID_COPY);
    m_ItemCut = m_PopupMenu.Append(wxID_CUT);
    m_ItemPast = m_PopupMenu.Append(wxID_PASTE);

	// Events
	Connect( wxEVT_PAINT, wxPaintEventHandler(DialogMdoEdit::OnPaint));

        // Tree
    m_tree->Connect( wxEVT_COMMAND_TREE_END_LABEL_EDIT, wxTreeEventHandler(DialogMdoEdit::OnTreeEndLabelEdit), NULL, this);
    m_tree->Connect( wxEVT_COMMAND_TREE_BEGIN_LABEL_EDIT, wxTreeEventHandler(DialogMdoEdit::OnTreeBeginLabelEdit), NULL, this);

    m_tree->Connect( wxEVT_COMMAND_TREE_END_DRAG, wxTreeEventHandler(DialogMdoEdit::OnTreeEndDrag), NULL, this);
    m_tree->Connect( wxEVT_COMMAND_TREE_BEGIN_DRAG, wxTreeEventHandler(DialogMdoEdit::OnTreeBeginDrag), NULL, this);

    m_tree->Connect( wxEVT_COMMAND_TREE_SEL_CHANGED, wxTreeEventHandler(DialogMdoEdit::OnTreeSelect), NULL, this);
    m_tree->Connect( wxEVT_COMMAND_TREE_ITEM_MENU, wxTreeEventHandler(DialogMdoEdit::OnTreeItemMenu), NULL, this);
    m_tree->Connect( wxEVT_COMMAND_TREE_ITEM_ACTIVATED, wxTreeEventHandler(DialogMdoEdit::OnTreeItemActivated), NULL, this);
    m_tree->Connect( wxEVT_COMMAND_TREE_KEY_DOWN, wxTreeEventHandler(DialogMdoEdit::OnTreeKeyDown), NULL, this);

        // Bts
    m_btAdd->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxEventHandler(DialogMdoEdit::OnAdd ), NULL, this);
    m_btRemove->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxEventHandler(DialogMdoEdit::OnRemove ), NULL, this);
    m_btRename->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxEventHandler(DialogMdoEdit::OnRename ), NULL, this);
    m_btEdit->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxEventHandler(DialogMdoEdit::OnEdit ), NULL, this);

        // Menu
    Connect( m_itemAdd->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxEventHandler(DialogMdoEdit::OnAdd ));
    Connect( m_itemRemove->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxEventHandler(DialogMdoEdit::OnRemove ));
    Connect( m_itemRename->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxEventHandler(DialogMdoEdit::OnRename ));
    Connect( m_itemEdit->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxEventHandler(DialogMdoEdit::OnEdit ));

        // Menu (suite)
    Connect( m_ItemUp->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxEventHandler(DialogMdoEdit::OnUp ));
    Connect( m_ItemDown->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxEventHandler(DialogMdoEdit::OnDown ));
    Connect( m_ItemCopy->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxEventHandler(DialogMdoEdit::OnCopy ));
    Connect( m_ItemCut->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxEventHandler(DialogMdoEdit::OnCut ));
    Connect( m_ItemPast->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxEventHandler(DialogMdoEdit::OnPast ));

    // Resize (win32)
    #if defined(__WXMSW__)
    Connect(wxEVT_SIZE, wxSizeEventHandler(DialogMdoEdit::OnResize ));
    #endif

    // Refresh
    BuildTree();
}
//------------------------------------------------------------------------------
#if defined(__WXMSW__)
void DialogMdoEdit::OnResize(wxSizeEvent &event)
{
    Refresh();

    event.Skip();
}
#endif
//------------------------------------------------------------------------------
void DialogMdoEdit::OnTreeBeginDrag(wxTreeEvent& event)
{
    /*wxTextDataObject my_data(_T("This text will be dragged."));
    wxDropSource dragSource( m_tree );
	dragSource.SetData( my_data );
	wxDragResult result = dragSource.DoDragDrop();*/

    /*wxDataObjectSimple dndData(wxDataFormat(_T("devisMaker_tree_mdo_data")));
    wxDropSource dragSource( m_tree );
	dragSource.SetData( dndData );
	wxDragResult result = dragSource.DoDragDrop();*/

    // need to explicitly allow drag
    if ( event.GetItem() != m_mainMdoSizer->GetTreeItemId() )
    {
        if(! m_mainMdoSizer->SearchTreeItemId( event.GetItem(), m_draggedMdo))
        {
            m_draggedMdo = NULL;
            event.Veto();
        }
        else
        {
            event.Allow();
        }
    }
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnTreeEndDrag(wxTreeEvent& event)
{
    mdo::DrawableObject *destMdo;
    if( !m_mainMdoSizer->SearchTreeItemId( event.GetItem(), destMdo))
    {
        return;
    }

    MoveCopyMdo(m_draggedMdo, destMdo, wxGetKeyState(WXK_CONTROL));
}
//------------------------------------------------------------------------------
bool DialogMdoEdit::VerifName(const wxString &name) const
{
    /*for(int i = 0; i < name.Len(); i++)
    {
        wxChar letter = name[i];

        // a-z
        if(letter >= 'a' && letter <= 'z')
        {
            continue;
        }

        // A-Z
        if(letter >= 'A' && letter <= 'Z')
        {
            continue;
        }

        // 0-9
        if(letter >= '0' && letter <= '9')
        {
            continue;
        }

        // _ / -
        if(letter == '-' || letter == '_')
        {
            continue;
        }

        return false;
    }*/

    return true;

}
//------------------------------------------------------------------------------
void DialogMdoEdit::MoveCopyMdo(mdo::DrawableObject *srcMdo, mdo::DrawableObject *destMdo, const bool &copy, const bool &addInIfsizer, const bool &addAfterDest)
{
    // Cas : Si on a déposé le mdo la ou on l'avait pris  ou dans son sizer actuel (rien à faire), sauf si copie
    if(!copy && (destMdo == srcMdo || destMdo == srcMdo->GetParent()))
    {
        return;
    }

    // Cas : Si on a déposé le mdo dans ses fils ==> Impossible, sauf pour les copies.
    if(!copy && destMdo->IsChildOfSizer(srcMdo))
    {
        return;
    }

    mdo::Sizer *srcSizer = static_cast<mdo::Sizer*>(srcMdo->GetParent());
    mdo::Sizer *destSizer;


    // Cas : Si on a déposé le mdo dans un sizer directement ==> à la fin. (si demandé)
    bool selectEndOfSizer;
    if(addInIfsizer && destMdo->GetType() == MDO_SIZER_TYPE_NAME)
    {
        destSizer = static_cast<mdo::Sizer*>(destMdo);
        destMdo = NULL;
        selectEndOfSizer = true;
    }
    else
    {
        destSizer = static_cast<mdo::Sizer*>(destMdo->GetParent());
        selectEndOfSizer = false;
    }

    if(copy) // Copie
    {
        //std::string newName = srcMdo->GetHumanName();//= GetUniqueMdoName(srcMdo->GetConfigName() + std::string("_"), srcMdo->GetType());
        destSizer->CopyMdo(srcMdo, destMdo, addAfterDest);
    }
    else
    {
        srcSizer->UnlinkMdo(srcMdo);
        destSizer->ReceiveMdo(srcMdo, destMdo, addAfterDest);
    }

    BuildTree();
    // Si on a déposé le mdo dans un sizer directement ==> à la fin. (si demandé)
    if(selectEndOfSizer)
    {
        m_tree->SelectItem(destSizer->GetMdoPtr(destSizer->GetMdoCount() - 1)->GetTreeItemId());
    }
    else if(addAfterDest)
    {
        m_tree->SelectItem(destSizer->GetMdoPtr(destSizer->GetMdoPos(destMdo) + 1)->GetTreeItemId());
    }
    else
    {
        m_tree->SelectItem(destSizer->GetMdoPtr(destSizer->GetMdoPos(destMdo) - 1)->GetTreeItemId());
    }
    RefreshBts();
    ForceRefreshDrawRect();
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnUp(wxEvent& event)
{
    bool isSelection = m_tree->GetSelection().IsOk();
    bool isRoot = isSelection && m_mainMdoSizer->GetTreeItemId() == m_tree->GetSelection();

    mdo::DrawableObject *mdo = NULL;
    if(isSelection && !isRoot && m_mainMdoSizer->SearchTreeItemId(m_tree->GetSelection(), mdo)) // Si trouvé
    {
        mdo::Sizer *sizer = static_cast<mdo::Sizer*>(mdo->GetParent());

        int pos = sizer->GetMdoPos(mdo);
        if(pos != 0) // Si on est pas au début
        {
            MoveCopyMdo(mdo, sizer->GetMdoPtr(pos - 1), false, false, false);
        }
    }
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnDown(wxEvent& event)
{
    bool isSelection = m_tree->GetSelection().IsOk();
    bool isRoot = isSelection && m_mainMdoSizer->GetTreeItemId() == m_tree->GetSelection();

    mdo::DrawableObject *mdo = NULL;
    if(isSelection && !isRoot && m_mainMdoSizer->SearchTreeItemId(m_tree->GetSelection(), mdo)) // Si trouvé
    {
        mdo::Sizer *sizer = static_cast<mdo::Sizer*>(mdo->GetParent());

        int pos = sizer->GetMdoPos(mdo);

        if(pos != sizer->GetMdoCount() - 1) // Si on est pas à la fin
        {
            MoveCopyMdo(mdo, sizer->GetMdoPtr(pos + 1), false, false, true);
        }
    }
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnCopy(wxEvent& event)
{
    bool isSelection = m_tree->GetSelection().IsOk();

    mdo::DrawableObject *mdo = NULL;
    if(isSelection && m_mainMdoSizer->SearchTreeItemId(m_tree->GetSelection(), mdo)) // Si trouvé
    {
        m_copyCutMdo = mdo;
        m_isCopy = true;
    }
    else
    {
        m_copyCutMdo = NULL;
    }

    RefreshBts();
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnCut(wxEvent& event)
{
    bool isSelection = m_tree->GetSelection().IsOk();
    bool isRoot = isSelection && m_mainMdoSizer->GetTreeItemId() == m_tree->GetSelection();

    mdo::DrawableObject *mdo = NULL;
    if(isSelection && !isRoot && m_mainMdoSizer->SearchTreeItemId(m_tree->GetSelection(), mdo)) // Si trouvé
    {
        m_copyCutMdo = mdo;
        m_isCopy = false;
    }
    else
    {
        m_copyCutMdo = NULL;
    }

    RefreshBts();
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnPast(wxEvent& event)
{
    bool isSelection = m_tree->GetSelection().IsOk();

    mdo::DrawableObject *destMdo = NULL;

    if(! (isSelection && m_copyCutMdo != NULL && ( m_isCopy || !destMdo->IsChildOfSizer(m_copyCutMdo)) && m_mainMdoSizer->SearchTreeItemId(m_tree->GetSelection(), destMdo))) // Si trouvé
    {
        return;
    }

    MoveCopyMdo(m_copyCutMdo, destMdo, m_isCopy, true);

    if(!m_isCopy)
    {
        m_copyCutMdo = NULL;
    }

    RefreshBts();
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnTreeBeginLabelEdit(wxTreeEvent& event)
{
    /*if(event.GetItem() == m_mainMdoSizer->GetTreeItemId()) // Si le root
    {
        wxMessageBox(_("You can't rename the root object."), _("Object Edit"), wxOK | wxICON_ERROR, this);
        event.Veto();
    }*/
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnTreeItemMenu(wxTreeEvent& event)
{
    PopupMenu(&m_PopupMenu, m_tree->GetPosition() + event.GetPoint());
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnTreeItemActivated(wxTreeEvent& event)
{
    m_tree->SelectItem(event.GetItem());
    wxCommandEvent tmp;
    OnEdit(tmp);
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnTreeSelect(wxTreeEvent& event)
{
    RefreshBts();
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnTreeEndLabelEdit(wxTreeEvent& event)
{
    if(event.IsEditCancelled())
    {
        event.Veto();
        return;
    }

    wxString newName = event.GetLabel();
    if( VerifName(newName))
    {
        mdo::DrawableObject *mdo = NULL;

        if(m_mainMdoSizer->SearchTreeItemId(event.GetItem(), mdo)) // Si trouvé
        {
            mdo->SetHumanName(StrWxToStd(newName));
        }
        else
        {
            event.Veto();
        }
    }
    else
    {
        //wxMessageBox(_("Object's names should contain a-Z, 0-9, \"-\" and \"_\" characters."), _("Object Edit"), wxOK | wxICON_ERROR, this);
        event.Veto();
    }
}
//------------------------------------------------------------------------------
DialogMdoEdit::~DialogMdoEdit()
{
    delete m_mainMdoSizer;
}
//------------------------------------------------------------------------------
void DialogMdoEdit::AddSizerToTree(const wxTreeItemId &item, mdo::Sizer *sizer)
{
    // On ajoute les elements
    for(int i = 0; i < sizer->GetMdoCount(); i++)
    {
        mdo::DrawableObject *currentMdo = sizer->GetMdoPtr(i);

        wxTreeItemId tempId = m_tree->AppendItem(item, StrStdToWx(currentMdo->GetHumanName()), GetTreeImagePos(currentMdo));
        // On indique au mdo à traiter son TreeItemId
        currentMdo->SetTreeItemId(tempId);

        // Cas d'un sizer ==> A developper
        if(currentMdo->GetType() == MDO_SIZER_TYPE_NAME)
        {
            AddSizerToTree(tempId, static_cast<mdo::Sizer*>(currentMdo));
        }
    }
}
//------------------------------------------------------------------------------
int DialogMdoEdit::GetTreeImagePos(const mdo::DrawableObject *mdo)
{
    std::string type = mdo->GetType();

    // Sizer
    if(mdo->GetType() == MDO_SIZER_TYPE_NAME)
    {
        if( static_cast<const mdo::Sizer*>(mdo)->IsVertical() )
        {
            type += "V";
        }
        else
        {
            type += "H";
        }
    }

    // Autre Mdos
    for(int i = 0; i < m_mdoTypeNameList.size(); i++)
    {
        if(type == m_mdoTypeNameList[i])
        {
            return i;
        }
    }
}
//------------------------------------------------------------------------------
void DialogMdoEdit::BuildTree()
{
    wxWindowUpdateLocker winLock(m_tree); // Bloque les redraw durant son temps de vie.

    // On vide
    m_tree->DeleteAllItems();

    // On ajoute
    wxTreeItemId root = m_tree->AddRoot( StrStdToWx(m_mainMdoSizer->GetHumanName()), GetTreeImagePos(m_mainMdoSizer));
    // On indique au sizer son TreeItemId
    m_mainMdoSizer->SetTreeItemId(root);
    AddSizerToTree(root, m_mainMdoSizer);

    // Tous visible
    m_tree->ExpandAll();

    RefreshBts();
}
//------------------------------------------------------------------------------
void DialogMdoEdit::RefreshBts()
{
    bool isSelection = m_tree->GetSelection().IsOk();
    bool isRoot = isSelection && m_mainMdoSizer->GetTreeItemId() == m_tree->GetSelection();

    bool isSizer = false;
    int posInSizer = 0;
    mdo::Sizer *parent = NULL;
    mdo::DrawableObject *mdo = NULL;
    if(m_mainMdoSizer->SearchTreeItemId(m_tree->GetSelection(), mdo)) // Si trouvé
    {
        isSizer = mdo->GetType() == MDO_SIZER_TYPE_NAME;
        if(!isRoot)
        {
            parent = static_cast<mdo::Sizer*>(mdo->GetParent());
            posInSizer = parent->GetMdoPos(mdo);
        }
    }

    m_btAdd->Enable(isSelection && isSizer);
    m_btRemove->Enable(isSelection && !isRoot);
    m_btRename->Enable(isSelection && !isRoot);
    m_btEdit->Enable(isSelection);

    // Menu
    m_itemAdd->Enable(m_btAdd->IsEnabled());
    m_itemRemove->Enable(m_btRemove->IsEnabled());
    m_itemRename->Enable(m_btRename->IsEnabled());
    m_itemEdit->Enable(m_btEdit->IsEnabled());

    // Menu (suite)
    m_ItemUp->Enable(isSelection && !isRoot && posInSizer > 0);
    m_ItemDown->Enable(isSelection && !isRoot && posInSizer < parent->GetMdoCount() - 1);

    m_ItemCopy->Enable(isSelection);
    m_ItemCut->Enable(isSelection && !isRoot);
    m_ItemPast->Enable(isSelection && m_copyCutMdo != NULL && ( m_isCopy || !mdo->IsChildOfSizer(m_copyCutMdo))); // SI pas une copie --> impossible de déplasser un mdo dans son enfant.
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnPaint(wxPaintEvent& event)
{
    // DC
    wxPaintDC realDc(this);
    wxPaintDC *dc = &realDc; // Habitude

    // Param du DC
    dc->SetBrush(*wxWHITE_BRUSH);
    dc->SetPen(*wxBLACK_PEN);
    double scale = 1;

    // Si page plus haute que large (portrait) ==> On augmente la hauteur par rapport au A4
    if(m_drawSpaceSizer->GetSize().GetHeight() >= m_drawSpaceSizer->GetSize().GetWidth())
    {
        scale = double(m_drawSpaceSizer->GetSize().GetWidth()) / double(PAGE_WIDTH);
        m_isPortrait = true;
    }
    else // Si non, on augmente la largeur.
    {
        scale = double(m_drawSpaceSizer->GetSize().GetHeight()) / double(PAGE_HEIGHT);
        m_isPortrait = false;
    }

    dc->SetUserScale(scale, scale);


    // Rect
    wxRect drawRect = wxRect( m_drawSpaceSizer->GetPosition(), m_drawSpaceSizer->GetSize() );
    drawRect.x *= 1.f / scale;
    drawRect.y *= 1.f / scale;
    drawRect.width *= 1.f / scale;
    drawRect.height *= 1.f / scale;

    wxRect paperRect = drawRect;

    // Marging
    drawRect.x += m_generalCfg->GetInt("pageMargingLeft");
    drawRect.y += m_generalCfg->GetInt("pageMargingTop");
    drawRect.width -= m_generalCfg->GetInt("pageMargingLeft") + m_generalCfg->GetInt("pageMargingRight");
    drawRect.height -= m_generalCfg->GetInt("pageMargingTop") + m_generalCfg->GetInt("pageMargingBottom");

    // Test
    if(drawRect != m_currentDrawRect) // Si le rect de dessin a changé...
    {
        // Et si c'est la taille qui a changé
        if(drawRect.GetSize() != m_currentDrawRect.GetSize())
        {
            // Re-calcul

            int width, height;
            if(m_mainMdoSizer->GetWidthProportion() == -1)
            {
                width = drawRect.GetWidth();
            }
            else
            {
                width = drawRect.GetWidth() * m_mainMdoSizer->GetWidthProportion();
            }

            if(m_mainMdoSizer->GetHeightProportion() == -1)
            {
                height = drawRect.GetHeight();
            }
            else
            {
                height = drawRect.GetHeight() * m_mainMdoSizer->GetHeightProportion();
            }

            m_mainMdoSizer->CalculSizeReal(dc, width, height);


            // Mode
            if(m_isPortrait)
            {
                m_labelMode->SetLabel(_("Mode : Portrait"));
            }
            else
            {
                m_labelMode->SetLabel(_("Mode : Landscape"));
            }
        }

        m_currentDrawRect = drawRect;
    }

    // Background
    dc->DrawRectangle(paperRect);

    // Content
    // Param du DC
    dc->SetBrush(*wxTRANSPARENT_BRUSH);
    dc->SetPen(wxPen( stdColorToWxColor(m_generalCfg->GetColor("tableColor")) ));
    m_mainMdoSizer->Draw(dc, m_currentDrawRect.GetPosition());

    // Liberartion du DC
    dc->SetBrush(wxNullBrush);
    dc->SetPen(wxNullPen);
}
//------------------------------------------------------------------------------
/*std::string DialogMdoEdit::GetUniqueMdoName(const std::string &prefix, const std::string &type)
{
    int suffix = 1;
    while(mdo::DrawableObjectExistsInCfg(m_generalCfg, type, prefix + nbToString(suffix)))
    {
        suffix++;
    }
    return prefix + nbToString(suffix);
}*/
//------------------------------------------------------------------------------
    // Bts
void DialogMdoEdit::OnAdd(wxEvent& event)
{
    try
    {
        bool isSelection = m_tree->GetSelection().IsOk();
        bool isSizer = false;

        mdo::DrawableObject *mdo = NULL;
        if(m_mainMdoSizer->SearchTreeItemId(m_tree->GetSelection(), mdo) && mdo->GetType() == MDO_SIZER_TYPE_NAME) // Si trouvé et sizer
        {
            const int typeNb = 5;
            std::string typeTbl[typeNb] = {MDO_IMAGE_TYPE_NAME, MDO_SIZER_TYPE_NAME, MDO_SPACER_TYPE_NAME, MDO_STATIC_TEXT_TYPE_NAME, MDO_DEVIS_TEXT_TYPE_NAME};
            wxString labelTbl[typeNb] = {_("Image"), _("Sizer (To organise objects)"), _("Spacer (To add gap)"), _("Static text"), _("Invoice's / Estimate's text")};

            int choice = wxGetSingleChoiceIndex(_("Select an object to add..."), _("Add object"), typeNb, labelTbl, this);
            if(choice != -1)
            {
                //std::string newName = GetUniqueMdoName(StrWxToStd(_("New_object_")), typeTbl[choice]);

                mdo::DrawableObject* newMdo = static_cast<mdo::Sizer*>(mdo)->CreateMdo(typeTbl[choice]);
                BuildTree();
                m_tree->SelectItem(newMdo->GetTreeItemId());
                m_tree->EditLabel(newMdo->GetTreeItemId());
            }
        }

        if(isSelection && isSizer);
        {
            ForceRefreshDrawRect();
            RefreshBts();
        }
    }
    catch(const std::exception& e) //Rattrape toutes les exceptions
    {
        wxMessageBox(_("Error : ") + wxString(_T("\n")) + StrStdToWx(e.what()), _("Error..."),  wxOK | wxICON_ERROR, this);
    }
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnRemove(wxEvent& event)
{
    try
    {
        bool isSelection = m_tree->GetSelection().IsOk();
        bool isRoot = isSelection && m_mainMdoSizer->GetTreeItemId() == m_tree->GetSelection();

        mdo::DrawableObject *mdo = NULL;
        if(m_mainMdoSizer->SearchTreeItemId(m_tree->GetSelection(), mdo)) // Si trouvé
        {
            if(isSelection && !isRoot)
            {
                mdo::Sizer *parent = static_cast<mdo::Sizer*>(mdo->GetParent());
                parent->EraseMdo(mdo);
                BuildTree();
                m_tree->SelectItem(parent->GetTreeItemId());
                ForceRefreshDrawRect();
            }
        }
    }
    catch(const std::exception& e) //Rattrape toutes les exceptions
    {
        wxMessageBox(_("Error : ") + wxString(_T("\n")) + StrStdToWx(e.what()), _("Error..."),  wxOK | wxICON_ERROR, this);
    }
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnTreeKeyDown(wxTreeEvent& event)
{
    bool isSelection = m_tree->GetSelection().IsOk();
    bool isRoot = isSelection && m_mainMdoSizer->GetTreeItemId() == m_tree->GetSelection();

    mdo::DrawableObject *mdo = NULL;
    if(m_mainMdoSizer->SearchTreeItemId(m_tree->GetSelection(), mdo)) // Si trouvé
    {
        if(isSelection && !isRoot)
        {
            if(event.GetKeyCode() == WXK_DELETE)
            {
                wxCommandEvent tmp;
                OnRemove(tmp);
            }

            event.Skip();
        }
    }
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnRename(wxEvent& event)
{
    bool isSelection = m_tree->GetSelection().IsOk();
    bool isRoot = isSelection && m_mainMdoSizer->GetTreeItemId() == m_tree->GetSelection();

    if(isSelection && !isRoot)
    {
        m_tree->EditLabel(m_tree->GetSelection());
    }
}
//------------------------------------------------------------------------------
void DialogMdoEdit::OnEdit(wxEvent& event)
{
    wxTreeItemId selection = m_tree->GetSelection();
    mdo::DrawableObject *mdo = NULL;
    if(! (selection.IsOk() && m_mainMdoSizer->SearchTreeItemId(m_tree->GetSelection(), mdo))) // Si trouvé
    {
        return;
    }

    try
    {
        DialogProperty MyDialogProperty(this, wxID_ANY, m_generalCfg, NULL);

        mdo->EditPropertyDialog(&MyDialogProperty);

        if(MyDialogProperty.ShowModal() == wxID_OK)
        {
            MyDialogProperty.SaveCfg();

            mdo->LoadFromCfg();
            BuildTree();
            m_tree->SelectItem(mdo->GetTreeItemId());
            ForceRefreshDrawRect();
            RefreshBts();
        }
    }
    catch(const std::exception& e) //Rattrape toutes les exceptions
    {
        wxMessageBox(_("Error : ") + wxString(_T("\n")) + StrStdToWx(e.what()), _("Error..."),  wxOK | wxICON_ERROR, this);
    }
}
//------------------------------------------------------------------------------
void DialogMdoEdit::ForceRefreshDrawRect()
{
    m_currentDrawRect = wxRect(0, 0, 0, 0); // Pour forcer le re-calcul
    Refresh();
}
//------------------------------------------------------------------------------
